#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>

#ifdef __MWERKS__	
#include "console.h"
#endif

/*prototypes*/
void CreateSequences(FILE *fv);

/*variables*/
char **names, **sequences;
int numTaxa, numSites;

/**************/
void CreateSequences(FILE *fv)
{
	int i;
	char str[255];

	fgets(str, 255, fv);
	if ( sscanf( str, " %d %d", &numTaxa, &numSites)!=2 ) {
		fprintf(stderr, "Error reading PHYLIP parameters from input\n");
		exit(0);
	}
	if ( (names=(char **)malloc(sizeof(char *)*numTaxa))==NULL ) {
		fprintf(stderr, "Out of memory allocating names\n");
		exit(0);
	}
	if ( (sequences=(char **)malloc(sizeof(char *)*numTaxa))==NULL ) {
		fprintf(stderr, "Out of memory allocating sequences\n");
		exit(0);
	}
	for(i=0;i<numTaxa;i++){
		if ( (names[i]=(char *)malloc(sizeof(char)*11))==NULL ) {
			fprintf(stderr, "Out of memory allocating names\n");
			exit(0);
		}
		if ( (sequences[i]=(char *)malloc(sizeof(char)*(numSites+1)))==NULL ) {
			fprintf(stderr, "Out of memory allocating sequences\n");
			exit(0);
		}
	}
	rewind(fv);
}


/********************************/
main(int argc, char **argv)
{
	int n1, n2, dset, n, b, i;
	char ch;
	char str[255];
	FILE *fv;

#ifdef __MWERKS__	
	argc=ccommand(&argv);
#endif

	fv=NULL;
	if (argc>1) {
		if ( (fv=fopen(argv[1], "rt"))==NULL ) {
			fprintf(stderr, "Can't open file, '%s'\n", argv[1]);
			exit(0);
		}
	}

	CreateSequences(stdin);
	
	fprintf(stdout, "#NEXUS\n\n");
	fprintf(stderr, "Sequences converted:\n");
	dset=0;
	do{
		fgets(str, 255, stdin);
		if ( sscanf( str, " %d %d", &n1, &n2)!=2 ) {
			fprintf(stderr, "Error reading parameters for dataset %d\n", dset+1);
			exit(0);
		}
		if ( n1!=numTaxa || n2!=numSites ) {
			fprintf(stderr, "Parameters for dataset %d don't match the first dataset\n", dset+1);
			exit(0);
		}
		n=0;
		do {
			ch=fgetc(stdin);
			while ( !feof(stdin) && isspace(ch)) 
				ch=fgetc(stdin);
				
			if ( feof(stdin) && n==0) {
				fprintf(stderr, "\n\nFinished!\n"); 
				exit(0);
			}
			if ( feof(stdin) ) {
				fprintf(stderr, "Unexpected end of file on standard input\n"); 
				exit(0);
			}
			if(n==0) {
				fprintf(stdout, "[Dataset %d]\n", dset+1);
				fprintf(stdout, "BEGIN DATA;\n\tDIMENSIONS NTAX=%d NCHAR=%d;\n\tFORMAT MISSING=? GAP=- DATATYPE=DNA;\n\tMATRIX\n", numTaxa, numSites);
			}
			i=0;
			while ( i<10 && !feof(stdin) && !isspace(ch) ) {
				names[n][i]=ch;
				ch=fgetc(stdin);
				i++;
			}
			while (i<10) {
				names[n][i]=' ';
				i++;
			}
			names[n][10]='\0';
			if (i==0) {
				fprintf(stderr, "Name missing for species %d\n", n+1);
				exit(0);
			}
			while (!feof(stdin) && i<10) {
				ch=fgetc(stdin);
				i++;
			}
			
			if ( feof(stdin) ) {
				fprintf(stderr, "Unexpected end of file on standard input\n");
				exit(0);
			}
			
			b=0;
			while ( !feof(stdin) && b<numSites) {
				if ( !isspace(ch) ) {
					sequences[n][b]=ch;
					b++;
				}
				ch=toupper(fgetc(stdin));
			}
			sequences[n][b]='\0';
			if ( b<numSites ) {
				fprintf(stderr, "Unexpected end of file on standard input\n");
				exit(0);
			}
			n++;
			
			if ( n<numTaxa && feof(stdin) ) {
				fprintf(stderr, "Too few sequences in input file\n");
				exit(0);
			}
		} while ( n<numTaxa );
		for(n=0;n<numTaxa;n++)
			fprintf(stdout, "%s %s\n", names[n], sequences[n]);
		fprintf(stdout, "\t;\nEND;\n\n");
		
		if (fv!=NULL) {
			ch=fgetc(fv);
			while (!feof(fv)) {
				fputc(ch, stdout);
				ch=fgetc(fv);
			}
			fputc('\n', stdout);
			rewind(fv);
		}
		
		fputc('.', stderr);
		dset++;
		if (dset%50==0)
			fputc('\n', stderr);
	} while(!feof(stdin));
	
	if (fv!=NULL)
		fclose(fv);
}
